
/**
 * <p>Title: Italia Stemmer</p>
 * <p>Description: Italian Stemmer for CAT Module in Bizon Project</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company:Bizon Project</p>
 * @author Michele Nannipieri
 * @version 1.0
 */

/*
The stemming algorithm

Defining R1, R2 and RV
All the stemmers make use of at least one of the region definitions R1 and R2.
They are defined as follows:
R1 is the region after the first non-vowel following a vowel,
  or is the null region at the end of the word if there is no such non-vowel.

R2 is the region after the first non-vowel following a vowel in R1,
  or is the null region at the end of the word if there is no such non-vowel.

The definition of vowel varies from language to language. In French, for example,
   is a vowel, and in Italian i between two other vowels is not a vowel.
  The class of letters that constitute vowels is made clear in each stemmer.

Below, R1 and R2 are shown for a number of English words,
    b   e   a   u   t   i   f   u   l
                      |<------------->|    R1
                              |<----->|    R2
Letter t is the first non-vowel following a vowel in beautiful, so R1 is iful.
  In iful, the letter f is the first non-vowel following a vowel, so R2 is ul.
    b   e   a   u   t   y
                      |<->|    R1
                        ->|<-  R2
In beauty, the last letter y is classed as a vowel.
Again, letter t is the first non-vowel following a vowel, so R1 is just the last letter, y.
 R1 contains no non-vowel, so R2 is the null region at the end of the word.
    b   e   a   u
                ->|<-  R1
                ->|<-  R2
In beau, R1 and R2 are both null.

Other examples:
    a   n   i   m   a   d   v   e   r   s   i   o   n
          |<----------------------------------------->|    R1
                  |<--------------------------------->|    R2

    s   p   r   i   n   k   l   e   d
                      |<------------->|    R1
                                    ->|<-  R2

    e   u   c   h   a   r   i   s   t
              |<--------------------->|    R1
                          |<--------->|    R2

If the second letter is a consonant, RV is the region after the next following vowel,
 or if the first two letters are vowels, RV is the region after the next consonant,
 and otherwise (consonant-vowel case) RV is the region after the third letter.
 But RV is the end of the word if these positions cannot be found.

For example,
    m a c h o     o l i v a     t r a b a j o      u r e o
         |...|         |...|         |.......|         |...|


Algorithm

Italian can include the following accented forms:
                           
First, replace all acute accents by grave accents.
And, as in French, put u after q, and u, i between vowels into upper case.
The vowels are then
a   e   i   o   u               

Always do steps 0 and 1.

Step 0: Attached pronoun
Search for the longest among the following suffixes
ci   gli   la   le   li   lo   mi   ne   si   ti   vi
sene   gliela   gliele   glieli   glielo   gliene   mela   mele   meli   melo
mene   tela   tele   teli   telo   tene   cela   cele   celi   celo   cene
vela   vele   veli   velo   vene

following one of
(a) ando   endo
(b) ar   er   ir

in RV. In case of (a) the suffix is deleted, in case (b) it is replace by e
(guardandogli -> guardando, accomodarci -> accomodare)

Step 1: Standard suffix removal
izione izioni itore itori itrice itrici
  delete if in RV
Search for the longest among the following suffixes, and perform the action indicated.
anza   anze   ico   ici   ica   ice   iche   ichi   ismo   ismi   abile   abili
ibile   ibili   ista   iste   isti   ist   ist   ist   oso   osi   osa   ose
mente   atrice   atrici
  delete if in R2
azione   azioni   atore   atori
  delete if in R2
logia   logie
  replace with log if in R2
uzione   uzioni   usione   usioni
  replace with u if in R2
enza   enze
  replace with ente if in R2
amento   amenti   imento   imenti
  delete if in RV
amente
  delete if in R1
  if preceded by iv, delete if in R2 (and if further preceded by at, delete if in R2),
  otherwise, if preceded by os, ic or abil, delete if in R2
it
  delete if in R2
ivo   ivi   iva   ive
  delete if in R2

Do step 2 if no ending was removed by step 1.

Step 2: Verb suffixes
Search for the longest among the following suffixes in RV, and if found, delete.
ammo   ando   ano   are   arono   asse   assero   assi   assimo   ata   ate   ati
ato   ava   avamo   avano   avate   avi   avo   emmo   enda   ende   endi   endo
er   erai   eranno   ere   erebbe   erebbero   erei   eremmo   eremo   ereste
eresti   erete   er   erono   essero   ete   eva   evamo   evano   evate   evi
evo   Yamo   iamo   immo   ir   irai   iranno   ire   irebbe   irebbero   irei
iremmo   iremo   ireste   iresti   irete   ir   irono   isca   iscano   isce
isci   isco   iscono   issero   ita   ite   iti   ito   iva   ivamo   ivano
ivate   ivi   ivo   ono   uta   ute   uti   uto   ar   er

Always do steps 3a and 3b.
Step 3a
Delete a final a, e, i, o, , ,  or  if it is in RV,
and a preceding i if it is in RV (crocchi -> crocch, crocchio -> crocch)
Step 3b
Replace final ch (or gh) with c (or g) if in RV (crocch -> crocc)
Finally,
	Turn I and U back into lower case
*/

package italianstemmer;

public class ItalianStemmer {

  public ItalianStemmer() {
  }

  /* String Cleaning  */
  private String Clean( String str ) {
     int last = str.length();
     Character ch = new Character( str.charAt(0) );
     String temp = "";
     for ( int i=0; i < last; i++ ) {
         if ( ch.isLetterOrDigit( str.charAt(i) ) )
            temp += str.charAt(i);
     }
     return temp;
  } //clean


  /* Vowel elaboration and changement */
  private String changevowel(String str) {
    char ch;
    String tmp = "";
    for ( int i=0; i<str.length(); i++ ) {
      ch=str.charAt(i);
      switch ( ch ) {
        case '': case '':
          ch='';
          break;
        case '': case '':
          ch='';
          break;
        case '': case '':
          ch='';
          break;
        case '': case '':
          ch='';
          break;
        case '': case '':
          ch='';
          break;
        case 'u':
          if (i>0) {
            if (str.charAt(i-1)=='q') { ch='U'; break;}
            if (i<(str.length()-1)) {
              if (vowel(str.charAt(i-1)) && vowel (str.charAt(i+1))) {ch='U';break;}
            }
          }
        case 'i':
          if (i>0) {
            if (str.charAt(i-1)=='q') { ch='I'; break;}
            if (i<(str.length()-1)) {
              if (vowel(str.charAt(i-1)) && vowel (str.charAt(i+1))) {ch='I';break;}
            }
          }
        default:  /* a,e,o, other chars */
          break;
      }
      tmp += ch;
    }
    return tmp;
  }

  /* check vowel presence */
  private boolean vowel(char ch) {
     switch ( ch ) {
        case 'a': case '': case '':
        case 'e': case '': case '':
        case 'i': case '': case '':
        case 'o': case '': case '':
        case 'u': case '': case '':
          return true;
        default :
          return false;
     }
  }


   /* Delete word prefixes */
  private String stripPrefixes ( String str) {
     String[] prefixes = { "kilo", "micro", "milli", "intra", "ultra", "mega", "nano", "pico", "pseudo"};
     int last = prefixes.length;
     for ( int i=0 ; i<last; i++ ) {
         if ( str.startsWith( prefixes[i] ) ) {
            String temp = "";
            for ( int j=0 ; j< str.length()-prefixes[i].length(); j++ )
                temp += str.charAt( j+prefixes[i].length() );
            return temp;
         }
     }
     return str;
  }

  /* Calculate R1 word index */
  private int R1calc( String str) {
    int posc=0;
    for (int i=0;i<str.length();i++) {
        if (!vowel(str.charAt(i))) {
          if  ( (i>0) && (vowel(str.charAt(i-1))) ) {
            if (i<(str.length()-1)) posc=(i+1);
            break;
          }
        }
    }
    return posc;
  }

/* Calculate R2 word index */
  private int R2calc( String str, int R1) {
    int posc=0;
    for (int i=R1;i<str.length();i++) {
        if (!vowel(str.charAt(i))) {
          if  ( (i>0) && (vowel(str.charAt(i-1))) ) {
            if (i<(str.length()-1)) posc=(i+1);
            break;
          }
        }
    }
    return posc;
  }


  /* Calculate the RV word index */
  private int RVcalc( String str){
   int pos=0;
   if (str.length()<3) return pos;
   if (!vowel(str.charAt(1)) ) {
      for (int i=2; i<str.length(); i++) {
        if (vowel(str.charAt(i))) {
          if (i<(str.length()-1)) pos=(i+1);
          return pos;
        }
      }
    }

    if ( vowel(str.charAt(0)) && vowel(str.charAt(1)) ) {
      for (int i=2; i<str.length(); i++) {
        if (!vowel(str.charAt(i))) {
          if (i<(str.length()-1)) pos=(i+1);
          return pos;
        }
      }
    }

    if ( !vowel(str.charAt(0)) && vowel(str.charAt(1)) ) {
      pos=3;
      return pos;
    }

    return pos;
 }

 /* elaborate step -1 - irregular verbs
  only irregular forms for each verb
  presente indicativo, imperfetto, presente congiuntivo,
  presente condizionale, passato remoto, imperativo, participio passato */
 private String stepminus1(String str) {

    String irrverbs[][]={
      {"aborrire","aborrisco","aborrisci","aborrisce","aborriscono","aborrii","aborrito"},
      {"accendere","accesi","accese","accesero","acceso","accesa","accese","accesi"},
      {"accludere","acclusi","accluse","acclusero","accluso","acclusa","acclusi","accluse"},
      {"accorgersi","accorsi","accorse","accorsero","accorto","accorta","accorti","accorte"},
      {"affiggere","affissi","affisse","affissero","affisso","affissa","affissi","affisse","affiggente","affiggenti"},
      {"affliggere","afflissi","afflisse","afflissero","afflitto","afflitta","afflitti",
       "afflitte"},
      {"alludere","allusi","alluse","allusero","alluso","alludente","alludenti"},
      {"andare","vado","vai","va","andiamo","andate","vanno","andai","and","andato",
       "andata","andati","andate","nadnte","andanti"},
      {"annettere","annettei","annett","annesso","annessione","annettente","annettenti"},
      {"apparire","apparisco","apparisci","appare","appariscono","appaio","apparii","apparvi",
       "apparve","apparvero","apparso","apparsa","apparse","apparsi","apparente","apparenti"},
      {"appendere","appesi","appese","appesero","appeso","appesa","appesi","appese"},
      {"aprire","aprii","apr","aperto","aperta","aperti","aperte","apertura","aperture"},
      {"ardere","arsi","arse","arsero","arso","arsa","arsi","arse","ardente","ardenti"},
      {"aspergere","aspersi","asperse","aspersero","asperso"},
      {"assalire","assalgo","assalgono","assalisco","assalisci","assalisce","assaliscono",
       "assalito","assalita","assaliti","assalite","assalente","assalenti"},
      {"assistere","assistei","assist","assisterono","assistetti","assistette","assistettero",
       "assistito","assistita","assistiti","assistite","assistente","assistenti"},
      {"assolvere","assolsi","assolse","assolsero","assolvei","assolv","assolvettero",
       "assolto","assolta","assolti","assolte"},
      {"assumere","assunsi","assunse","assunsero","assunto","assunta","assunti","assunte",
       "assumente","assumenti"},

      {"benedire","benedico","benedici","benedice","bendiciamo","benedicono","benedissi",
       "benedisti","benedisse","benedicemmo","benediceste","benedissero","benedetto",
       "benedetta","benedetti","benedette","benedicente","benedicenti"},
      {"bere","bevo","bevi","beve","beviamo","bevete","bevono","bevvi","bevetti","bevesti",
       "bevevo","bevevi","beveva","bevevamo","bevevate","bevevono",
       "bevve","bevette","bevemmo","beveste","bevvero","bevuto","bevuta","bevuti","bevute",
       "berr","berrai","berr","berremo","berrete","berranno","beva","beviate","bevano",
       "bevessi","bevesse","bevessimo","beveste","bevessero"},

       {"cadere","caddi","cadde","caddero","cadr","cadrai","cadr","cadremo","cadrete",
        "cadranno","caduto","caduta","caduti","cadute","cadente","cadenti"},
       {"cedere","cedetti","cedette","cedettero","ceduto","ceduta","ceduti","cedute","cedente","cedenti"},
       {"chiedere","chiesi","chiese","chiesero","chieda","chiediate","chiedano",
        "chiesto","chiesta","chiesti","chieste","chiedente","chiedenti"},
       {"chiudere","chiusi","chiuse","chiusero","chiuso","chiusa","chiusi","chiuse"},
       {"cingere","cinsi","cinse","cinsero","cinto","cinta","cinte","cinti"},
       {"cogliere","colgo","colgono","colsi","colse","colsero","colto","colta","colti","colte"},
       {"coincidere","coincisi","coincise","coincisero","coinciso"},
       {"comparire","comparisco","comparisci","compare","compariscono","compaio","comparii","comparvi",
        "comparve","comparvero","comparso","comparsa","comparse","comparsi","comparente","comparenti"},
       {"compiere","compii","comp","compiuto","compiuta","compiuti","compiute"},
       {"comprimere","compressi","compresse","compressero","compresso","compressa"},
       {"concedere","concessi","concesse","concessero","concessa","concesso"},
       {"condurre","conduco","conduci","conduce","conduciamo","conducete","conducono",
        "conducevo","conducevi","conduceva","conducevamo","conducevate","conducevano",
        "condussi","conducesti","condusse","conducemmo","conduceste","condussero",
        "conduca","conduciate","conducano","condotto","condotta","condotti","condotte",
        "conducente","conducenti"},
       {"confondere","confusi","confuse","confusero","confusa","confuse","cofondente","confondenti"},
       {"connettere","connessi","connesse","connessero","connessa","connesso",
        "connessione","connessioni","connettente","connettenti"},
       {"conoscere","conobbi","conobbe","conobbero","conosciuto","conosciuta","conosciuti",
        "conosciute","conoscente","conoscenti"},
       {"consistere","consistei","consistesti","consistette","consistemmo","consisteste",
        "consistettero","consistito","consistita","consistiti","consistite","consistente",
        "consistenza","consistenze","consistenti"},
       {"contundere","contusi","contuse","contusero","contuso","contusa","contundente","contundenti"},
       {"convergere","conversi","converse","conversero","converso","conversa","conversi",
        "converse","convergente","convergenti"},
       {"coprire","coprii","copristi","copr","copriste","coperto","coperta","coperti",
        "coperte","coprente","coprenti"},
       {"correggere","corressi"}



    };

    int lastr=irrverbs.length;

    for (int i=0;i<lastr;i++) {

      int lastc=irrverbs[i].length;

      for (int j=0;j<lastc;j++) {
        if (str.startsWith(irrverbs[i][j])) {
          String temp="";
          temp=irrverbs[i][0];
          return temp;
        }
      }
    }

    return str;
 }

 /* elaborate step 0 - attached pronoun */
  private String step0(String str, int RV) {
     String[] suffixes3 = {"ci", "gli", "la","le","li","lo","mi","ne","si","ti","vi","sene",
                           "gliela","gliele","glieli","glielo","gliene","mela","mele","meli",
                           "melo","mene","tela","tele","teli","telo","tene","cela","cele","celi",
                           "celo","cene","vela","vele","veli","velo","vene" };
     String[] suffixes1 = {"ando","endo"};
     String[] suffixes2 = {"ar","er","ir"};


     for ( int index1 = 0 ; index1 < suffixes1.length; index1++ ) {
       for ( int index3 = 0 ; index3 < suffixes3.length; index3++ ) {

          String tmp="", tmp2="";
          tmp += suffixes1[index1];
          tmp += suffixes3[index3];

          if ( (str.length()-RV) > tmp.length() ) {
            if (str.endsWith(tmp)) {
              for (int i=0; i<(str.length()-suffixes3[index3].length()); i++) {
                tmp2 += str.charAt(i);
              }
              return tmp2;
            }
          }
       }
     }

     for ( int index2 = 0 ; index2 < suffixes2.length; index2++ ) {
       for ( int index3 = 0 ; index3 < suffixes3.length; index3++ ) {

          String tmp="", tmp2="";
          tmp += suffixes2[index2];
          tmp += suffixes3[index3];

          if ( (str.length()-RV) > tmp.length() ) {
            if (str.endsWith(tmp)) {
              for (int i=0; i<(str.length()-suffixes3[index3].length()); i++) {
                tmp2 += str.charAt(i);
              }
              tmp2 += 'e';
              return tmp2;
            }
          }
        }
      }

      return str; /* case else */
  }

 /* elaborate step 1 - standard suffix removal */
  private String step1(String str, int R1, int R2, int RV) {
     String[] suffixes0 = {"izione","izioni","itore","itori","itrice","itrici"};
     String[] suffixes1 = {"anza", "anze", "ico","ici","ica","ice","iche","ichi",
                           "ismo","ismi","abile","abili","ibile","ibili","ista",
                           "iste","isti","ist","ist","ist","oso","osi","osa",
                           "ose","mente","atrice","atrici"};
     String[] suffixes2 = {"azione","azioni","atore","atori","ienza","ienze"};
     String[] suffixes3 = {"logia","logie"};
     String[] suffixes4 = {"uzione","uzioni","usione","usioni","uttore","uttori","utrice","utrici"};
     String[] suffixes5 = {"enza","enze"};
     String[] suffixes6 = {"amento","amenti","imento","imenti"};
     String[] suffixes7 = {"amente"};
     String[] suffixes8 = {"it"};
     String[] suffixes9 = {"ivo","ivi","iva","ive"};


      /* analysis -1.0- */
    for ( int index0 = 0 ; index0 < suffixes0.length; index0++ ) {
      String tmp="", tmp2="";
      tmp += suffixes0[index0];
      if ( (str.length()-RV+1) > tmp.length() ) {
        if (str.endsWith(tmp)) {
          for (int i=0; i<(str.length()-suffixes0[index0].length()); i++) {
            tmp2 += str.charAt(i);
          }
          return tmp2;
        }
      }
    }

  /* analysis -1.1- */
     int segnala=-1;
     int lunga=0;
     for ( int index1 = 0 ; index1 < suffixes1.length; index1++ ) {
          String tmp="";
          tmp += suffixes1[index1];
          if ( (str.length()-R2+1) > tmp.length() ) {
            if (str.endsWith(tmp)) {
              if (tmp.length()>lunga) {
                segnala=index1;  /* search the longest suffix */
                lunga=tmp.length();
              }
            }
          }
     }
     if (segnala >= 0) {
      String tmp2="";
      for (int i=0; i<(str.length()-suffixes1[segnala].length()); i++) {
        tmp2 += str.charAt(i);
      }
      return tmp2;
    }

  /* analysis -1.2- */
    for ( int index2 = 0 ; index2 < suffixes2.length; index2++ ) {
      String tmp="", tmp2="";
      tmp += suffixes2[index2];
      if ( (str.length()-R2+1) > tmp.length() ) {
        if (str.endsWith(tmp)) {
          for (int i=0; i<(str.length()-suffixes2[index2].length()); i++) {
            tmp2 += str.charAt(i);
          }
          return tmp2;
        }
      }
    }

  /* analysis -1.3- */
    for ( int index3 = 0 ; index3 < suffixes3.length; index3++ ) {
      String tmp="", tmp2="";
      tmp += suffixes3[index3];
      if ( (str.length()-R2+1) > tmp.length() ) {
        if (str.endsWith(tmp)) {
          for (int i=0; i<(str.length()-suffixes3[index3].length()); i++) {
            tmp2 += str.charAt(i);
          }
          tmp2 += "log";
          return tmp2;
        }
      }
    }

  /* analysis -1.4- */
    for ( int index4 = 0 ; index4 < suffixes4.length; index4++ ) {
      String tmp="", tmp2="";
      tmp += suffixes4[index4];
      if ( (str.length()-R1+1) > tmp.length() ) {
        if (str.endsWith(tmp)) {
          for (int i=0; i<(str.length()-suffixes4[index4].length()); i++) {
            tmp2 += str.charAt(i);
          }
          tmp2 += 'u';
          return tmp2;
        }
      }
    }

  /* analysis -1.5- */
    for ( int index5 = 0 ; index5 < suffixes5.length; index5++ ) {
      String tmp="", tmp2="";
      tmp += suffixes5[index5];
      if ( (str.length()-R2+1) > tmp.length() ) {
        if (str.endsWith(tmp)) {
          for (int i=0; i<(str.length()-suffixes5[index5].length()); i++) {
            tmp2 += str.charAt(i);
          }
          tmp2 += "ente";
          return tmp2;
        }
      }
    }

  /* analysis -1.6- */
    for ( int index6 = 0 ; index6 < suffixes6.length; index6++ ) {
      String tmp="", tmp2="";
      tmp += suffixes6[index6];
      if ( (str.length()-RV+1) > tmp.length() ) {
        if (str.endsWith(tmp)) {
          for (int i=0; i<(str.length()-suffixes6[index6].length()); i++) {
            tmp2 += str.charAt(i);
          }
          return tmp2;
        }
      }
    }

  /* analysis -1.7- */
    for ( int index7 = 0 ; index7 < suffixes7.length; index7++ ) {
      String tmp="", tmp2="";
      tmp += suffixes7[index7];
      if ( (str.length()-R1+1) > tmp.length() ) {
        if (str.endsWith(tmp)) {
          for (int i=0; i<(str.length()-suffixes7[index7].length()); i++) {
            tmp2 += str.charAt(i);
          }
          return tmp2;
        }
      }
    }

  /* analysis -1.8- */
    for ( int index8 = 0 ; index8 < suffixes8.length; index8++ ) {
      String tmp="", tmp2="";
      tmp += suffixes8[index8];
      if ( (str.length()-R2+1) > tmp.length() ) {
        if (str.endsWith(tmp)) {
          for (int i=0; i<(str.length()-suffixes8[index8].length()); i++) {
            tmp2 += str.charAt(i);
          }
          return tmp2;
        }
      }
    }

      /* analysis -1.9- */
    for ( int index9 = 0 ; index9 < suffixes9.length; index9++ ) {
      String tmp="", tmp2="";
      tmp += suffixes9[index9];
      if ( (str.length()-R2+1) > tmp.length() ) {
        if (str.endsWith(tmp)) {
          for (int i=0; i<(str.length()-suffixes9[index9].length()); i++) {
            tmp2 += str.charAt(i);
          }
          return tmp2;
        }
      }
    }


     return str; /* case else */
  }

 /* elaborate step 2 - verb suffix removal */
  private String step2(String str, int RV) {
     String[] suffixes1 = {"ammo", "ando", "ano","are","arono","asse","assero","assi",
                           "assimo","ata","ate","ati","ato","ava","avamo","avano",
                           "avate","avi","avo","ante","anti",
                           "emmo","endo","er","erai","eranno",
                           "ere","erebbe","erebbero","erei","eremmo","eremo","ereste",
                           "eresti","erete","er","erono","essero","ete","eva",
                           "evamo","evano","evate","evi","evo",
                           "yamo","iamo","immo","ir","irai","iranno","ire","irebbe",
                           "irei","iremmo","iremo","ireste","iresti","irete","ir",
                           "irono","isca","iscano","isce","isci","isco","iscono",
                           "issero","ita","ite","iti","ito","iva","ivamo","ivano",
                           "ivate","ivi","ivo","ono","uta","ute","uti","uto","ar","er","rre"};

     int segnala=-1;
     int lunga=0;
     for ( int index1 = 0 ; index1 < suffixes1.length; index1++ ) {
          String tmp="";
          tmp += suffixes1[index1];
          if ( (str.length()-RV) > tmp.length() ) {
            if (str.endsWith(tmp)) {
              if (tmp.length()>lunga) {
                segnala=index1;  /* search the longest suffix */
                lunga=tmp.length();
              }
            }
          }
     }
     if (segnala >= 0) {
      String tmp2="";
      for (int i=0; i<(str.length()-suffixes1[segnala].length()); i++) {
        tmp2 += str.charAt(i);
      }
      return tmp2;
    }

    return str;
  }

 /* elaborate step 3a - last vowel removal */
  private String step3a(String str, int RV) {
    if ( (str.length()-RV) > 1) {
      if (  (str.endsWith("a")) ||
            (str.endsWith("e")) ||
            (str.endsWith("i")) ||
            (str.endsWith("o")) ||
            (str.endsWith("")) ||
            (str.endsWith("")) ||
            (str.endsWith("")) ||
            (str.endsWith("")) ) {
        String tmp="";
        for (int i=0; i<(str.length()-1); i++) {
          tmp += str.charAt(i);
        }
        if (tmp.endsWith("i")) {
          String tmp2="";
          for (int i=0; i<(tmp.length()-1); i++) {
            tmp2 += tmp.charAt(i);
          }
          return tmp2;
        }

        return tmp;
      }
    }
    return str;
  }


 /* elaborate step 3b - removal ch and gh at the end of the word */
  private String step3b(String str, int RV) {
    if ( (str.length()-RV) > 2) {
      if ( (str.endsWith("ch")) || (str.endsWith("gh")) ) {
        String tmp="";
        for (int i=0; i<(str.length()-1); i++) {
          tmp += str.charAt(i);
        }
        return tmp;
      }
    }
    return str;
  }

 /* elaborate step 3c - replace U -> u and I -> i */
  private String step3c(String str) {
    String tmp="";
    tmp = str.toLowerCase();
    return tmp;
  }


   /**  Takes a String as input and returns its stem as a String.*/
  public String stripAffixes( String str ) {
    str = str.toLowerCase();
    str = Clean(str);
    str = changevowel(str);

    if (( str != "" ) && (str.length() > 2)) {
       str = stripPrefixes(str);
    }

    int R1,R2,RV;

    R1=R1calc(str);
    R2=R2calc(str,R1);
    RV=RVcalc(str);

    /* Step -1 - irregular verbs */
    str=stepminus1(str);

    /* Step 0 - attached pronoun */
    str= step0(str,RV);

    /* Step 1 - standard suffix removal */
    String strold;
    strold=str;
    str= step1(str,R1,R2,RV);

    /* Step 2 - verb suffixes removal only if any changement is from step 1 */
    if (str.equals(strold)) str=step2(str,RV);

    /* Step 3 - final replacement */
    str=step3a(str,RV);
    str=step3b(str,RV);
    str=step3c(str);

    return str;

 } //stripAffixes

/*
 // For testing, print the stemmed version of a word //
    public static void main(String[] args) throws IOException {
	String word = args[0];
        ItalianStemmer stemmer = new ItalianStemmer();
	String stem = stemmer.stripAffixes(word);
	System.out.println(stem);
    }
*/

} //class


